@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-gst.c,v 1.99 2004/10/25 16:12:05 rbultje Exp $
+ * $Id: bacon-video-widget-gst.c,v 1.106 2004/11/11 19:12:10 rbultje Exp $
  *
  */
 
@@ -45,7 +45,6 @@
 #include "baconvideowidget-marshal.h"
 #include "scrsaver.h"
 #include "video-utils.h"
-#include "gststreaminfo.h"
 #include "gstscreenshot.h"
 
 #define DEFAULT_HEIGHT 420
@@ -823,26 +822,33 @@ state_change (GstElement *play, GstElementState old_state,
     g_object_get (G_OBJECT (bvw->priv->play), "stream-info",
 		  &streaminfo, NULL);
     for ( ; streaminfo != NULL; streaminfo = streaminfo->next) {
-      GstStreamInfo *info = streaminfo->data;
+      GObject *info = streaminfo->data;
+      gint type;
+      GParamSpec *pspec;
+      GEnumValue *val;
 
-      switch (info->type) {
-        case GST_STREAM_TYPE_AUDIO:
-          bvw->priv->media_has_audio = TRUE;
-          break;
-        case GST_STREAM_TYPE_VIDEO: {
+      g_object_get (info, "type", &type, NULL);
+      pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (info), "type");
+      val = g_enum_get_value (G_PARAM_SPEC_ENUM (pspec)->enum_class, type);
+
+      if (strstr (val->value_name, "AUDIO")) {
+        bvw->priv->media_has_audio = TRUE;
+      } else if (strstr (val->value_name, "VIDEO")) {
+        GstPad *pad = NULL;
+
+        g_object_get (info, "object", &pad, NULL);
+        if (pad != NULL && GST_PAD_REALIZE (pad) != NULL) {
+          g_assert (GST_IS_PAD (pad));
 
           /* handle explicit caps as well - they're set later */
-          if (GST_PAD_CAPS (info->pad))
-            caps_set (G_OBJECT (info->pad), NULL, bvw);
+          if (GST_PAD_REALIZE (pad)->link != NULL && GST_PAD_CAPS (pad))
+            caps_set (G_OBJECT (pad), NULL, bvw);
           else
-            g_signal_connect (info->pad, "notify::caps",
+            g_signal_connect (pad, "notify::caps",
                 G_CALLBACK (caps_set), bvw);
-
-          bvw->priv->media_has_video = TRUE;
-          break;
         }
-        default:
-          break;
+
+        bvw->priv->media_has_video = TRUE;
       }
     }
   } else if (new_state <= GST_STATE_READY &&
@@ -940,6 +946,7 @@ bacon_video_widget_finalize (GObject * object)
       gst_tag_list_free (bvw->priv->tagcache);
       bvw->priv->tagcache = NULL;
     }
+  g_free (bvw->priv);
 }
 
 static void
@@ -1876,7 +1883,7 @@ bacon_video_widget_get_metadata_string (BaconVideoWidget * bvw,
     }
 
   if (res)
-    g_value_set_string (value, string);
+    g_value_take_string (value, string);
   else
     g_value_set_string (value, NULL);
 
@@ -1919,6 +1926,8 @@ bacon_video_widget_get_metadata_int (BaconVideoWidget * bvw,
       gint num, t;
 
       integer = 0;
+      if (bvw->priv->tagcache == NULL)
+        break;
       for (num = 0; ; num++) {
         if (!gst_tag_list_get_uint_index (bvw->priv->tagcache,
 					  GST_TAG_BITRATE, num, &t))
@@ -2063,11 +2072,21 @@ bacon_video_widget_get_current_frame (BaconVideoWidget * bvw)
   g_object_get (G_OBJECT (bvw->priv->play),
       "stream-info", &streaminfo, NULL);
   for (; streaminfo != NULL; streaminfo = streaminfo->next) {
-    GstStreamInfo *info = streaminfo->data;
+    GObject *info = streaminfo->data;
+    gint type;
+    GParamSpec *pspec;
+    GEnumValue *val;
 
-    if (info->type == GST_STREAM_TYPE_VIDEO) {
-      from = gst_caps_copy (GST_PAD_CAPS (info->pad));
-      break;
+    g_object_get (info, "type", &type, NULL);
+    pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (info), "type");
+    val = g_enum_get_value (G_PARAM_SPEC_ENUM (pspec)->enum_class, type);
+                                                                                
+    if (strstr (val->value_name, "VIDEO")) {
+      GstPad *pad;
+
+      g_object_get (info, "object", &pad, NULL);
+      g_assert (GST_IS_PAD (pad));
+      from = gst_caps_copy (GST_PAD_CAPS (pad));
     }
   }
   if (!from)
@@ -2129,12 +2148,23 @@ bacon_video_widget_error_quark (void)
   return q;
 }
 
+static void
+out_error (GstElement *el, GstElement *src,
+	   GError *err, gchar *debug,
+	   gpointer data)
+{
+  if (data && !*((GError **) data)) {
+    *((GError **) data) = g_error_copy (err);
+  }
+}
+
 GtkWidget *
 bacon_video_widget_new (int width, int height,
 			gboolean null_out, GError ** err)
 {
   BaconVideoWidget *bvw;
   GstElement *audio_sink = NULL, *video_sink = NULL;
+  gulong sig1, sig2;
 
   bvw = BACON_VIDEO_WIDGET (g_object_new
                             (bacon_video_widget_get_type (), NULL));
@@ -2173,6 +2203,27 @@ bacon_video_widget_new (int width, int height,
     audio_sink = gst_element_factory_make ("fakesink", "fakeaudiosink");
   }
 
+  sig1 = g_signal_connect (video_sink, "error", G_CALLBACK (out_error), err);
+  sig2 = g_signal_connect (audio_sink, "error", G_CALLBACK (out_error), err);
+  if (gst_element_set_state (video_sink,
+			     GST_STATE_READY) != GST_STATE_SUCCESS ||
+      gst_element_set_state (audio_sink,
+			     GST_STATE_READY) != GST_STATE_SUCCESS) {
+    if (err && !*err) {
+      g_set_error (err, 0, 0,
+		   "Failed to intialize %s output; check your configuration",
+		   GST_STATE (video_sink) == GST_STATE_NULL ?
+		   "video" : "audio");
+    }
+    gst_object_unref (GST_OBJECT (video_sink));
+    gst_object_unref (GST_OBJECT (audio_sink));
+    g_object_unref (G_OBJECT (bvw));
+    return NULL;
+  }
+  /* somehow, alsa hangs? */
+  gst_element_set_state (audio_sink, GST_STATE_NULL);
+  g_signal_handler_disconnect (video_sink, sig1);
+  g_signal_handler_disconnect (audio_sink, sig2);
   g_object_set (G_OBJECT (bvw->priv->play), "video-sink",
 		video_sink, NULL);
   g_object_set (G_OBJECT (bvw->priv->play), "audio-sink",
@@ -2185,9 +2236,9 @@ bacon_video_widget_new (int width, int height,
   g_signal_connect (G_OBJECT (bvw->priv->play), "state-change",
 		    G_CALLBACK (state_change), (gpointer) bvw);
   g_signal_connect (G_OBJECT (bvw->priv->play), "found_tag",
-		    (GtkSignalFunc) got_found_tag, (gpointer) bvw);
+		    G_CALLBACK (got_found_tag), (gpointer) bvw);
   g_signal_connect (G_OBJECT (bvw->priv->play), "error",
-		    (GtkSignalFunc) got_error, (gpointer) bvw);
+		    G_CALLBACK (got_error), (gpointer) bvw);
 
   /* We try to get an element supporting XOverlay interface */
   if (!null_out && GST_IS_BIN (bvw->priv->play)) {
