@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget.h,v 1.48 2004/09/21 11:28:18 hadess Exp $
+ * $Id: bacon-video-widget.h,v 1.49 2004/10/11 15:53:13 rbultje Exp $
  *
  * the xine engine in a widget - header
  */
@@ -27,6 +27,9 @@
 #include <gtk/gtkbox.h>
 #include <popt.h>
 
+/* for optical disc enumeration type */
+#include "totem-disc.h"
+
 G_BEGIN_DECLS
 
 #define BACON_VIDEO_WIDGET(obj)              (G_TYPE_CHECK_INSTANCE_CAST ((obj), bacon_video_widget_get_type (), BaconVideoWidget))
@@ -152,13 +155,6 @@ void bacon_video_widget_set_connection_speed     (BaconVideoWidget *bvw,
 						  int speed);
 int bacon_video_widget_get_connection_speed      (BaconVideoWidget *bvw);
 
-/* Optical devices */
-typedef enum {
-	MEDIA_DVD,
-	MEDIA_VCD,
-	MEDIA_CDDA
-} MediaType;
-
 void bacon_video_widget_set_media_device         (BaconVideoWidget *bvw,
 						  const char *path);
 gboolean bacon_video_widget_can_play             (BaconVideoWidget *bvw,
